/*
 * Copyright (c) 2003-2004 Jakub Jermar
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup arm32mm	
 * @{
 */
/** @file
 */ 


#ifndef __MM_H__
#define __MM_H__

#include "types.h"


#define FRAME_WIDTH					12 					/* 4KB frames */
#define FRAME_SIZE					(1 << FRAME_WIDTH)

#define PAGE_WIDTH					FRAME_WIDTH
#define PAGE_SIZE					FRAME_SIZE

#ifndef __ASM__
#	define KA2PA(x)					(((uintptr_t) (x)) - 0x80000000)
#	define PA2KA(x)					(((uintptr_t) (x)) + 0x80000000)
#else
#	define KA2PA(x)					((x) - 0x80000000)
#	define PA2KA(x)					((x) + 0x80000000)
#endif

#define PTL0_ENTRIES_ARCH 				(2<<12)				/* 4096 */
#define SET_PTL0_ADDRESS_ARCH(ptl0) 			( set_ptl0_addr((pte_level0_section *)(ptl0)) )

/** Frames per 1MB section */
#define FRAMES_PER_SECTION 				( ( 1 << 20 ) / FRAME_SIZE )

/** Converts adress to frame */
#define ADDR2PFN( addr ) 				( ((uintptr_t)(addr))>>FRAME_WIDTH )

/** Value of descriptor_type in page table entry that signals 
 * that this entry describe section
 */  
#define PTE_DESCRIPTOR_SECTION 				2

/**  Rights: user-no acess, kernel-read/write */
#define PTE_AP_USER_NO_KERNEL_RW 			1


/** Page table level 0 entry - section format */
typedef struct {
	unsigned descriptor_type     : 2; // should be 2
	unsigned bufferable          : 1;
	unsigned cacheable           : 1; 
   	unsigned machine_depend      : 1;
	unsigned domain              : 4;
	unsigned should_be_zero_1    : 1;
	unsigned access_permission   : 2; 	
	unsigned should_be_zero_2    : 8;
	unsigned section_base_addr   : 12;
} __attribute__ ((packed)) pte_level0_section;


/** Set adress of paging level 0 table
 * \param pt pointer to page table to set
 */   
static inline void set_ptl0_addr( pte_level0_section* pt) {
    asm volatile ( "mcr p15, 0, %0, c2, c0, 0 \n"
    :
    : "r"(pt)
    );
    
}

/** Page table that holds 1:1 mapping for kernel starting */ 
extern pte_level0_section page_table[PTL0_ENTRIES_ARCH];

/** Sets memory mapping for kernel */
void mm_kernel_mapping(void);
 
#endif

/** @}
 */
 
